%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This program computes the SMLE estimator for mixed logit model combining 
% RP and SP data and with unbalance panel data. Specifically, this code is
% tailored to such a case, in which each person has one RP observation (could be multinomial choice),
% and repeated SP observations (binary choice only). Extension for more general cases is 
% straightforward.
 
% Jia Yan, 11/05/2004
% Department of Logistics, Hong Kong Polytechnic University
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all;

% Define global variables (please see below for descriptions)
global REPTITION NUSERS CHOICE YVEC XFIXED XRANDOM NFC NNC NALT WEIGHTS...
       RCOEFFS HALTON NERROR RPDATA1 RPDATA2 SP RP TIMES DATAKINDS;  


% Do you want to use halton random draws? 1 if yes, 0 then use pseudo-random generator;
% generally, estimator using halton spequency has smaller variance.
HALTON = 1;

% REPTITION is the number of random draws used to approximate integration   
REPTITION = 1000;

% NUSERS is the number of persons
NUSERS = 548;

% NALT is the number of RP choice alternatives
NALT = 2;

% NFC is the number of varibles with fixed coefficients
NFC = 20;

% NNC is the number of variables with normal distributed coefficients
NNC = 4;

% RCOEFFS is the number of random parameters to be estimated, which may not equal to NNC
RCOEFFS = 4; 

%%%%%%%%%%%%%%%%%%%%%%% Load Data Here %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The data is in ASCII file, and it includes the following variables (columns)
% PID     -- NOBS by 1 vector of person's ID number;
% CHOICE  -- NOBS by 1 vector indlicating the RP alternatives. We use 0 to indicate SP choice;
% YVEC    -- NOBS by 1 binary vector indicating the chosen alternative (1 if chosen);   
% XFIXED  -- NOBS by NFC matrix containing all explanatory variables with fixed coefficients;
% XRANDOM -- NOBS by NNC matrix containing all explanatory variables with random (normal) coefficients;

% Binary variables indicating the different sources of data. For our example, by combining 2 RP with 1 SP,
% we then need
% RPDATA1 -- NOBS by 1 vector; 1 if the observation is from the first RP data;
% RPDATA2 -- NOBS by 1 vector; 1 if the observation is from the second RP data;
% RP      -- NOBS by 1 vector; 1 if the observation is from RPDATA1 or RPDATA2; 
% SP      -- NOBS by 1 vector; 1 if the observation is from SP data; 

% The above data can be saved in one file. You also need another ASCII file including 
% the following two variables (columns)
% TIMES   -- NUSERS by 1 vector indicating the number of observations for each person (sum of RP and SP);
% WEIGHTS -- NUSERS by 1 vector indicating the weights associated with each person in
%            the case of choice-based sampling;
% DATAKINDS -- NUSERS by 1 vector indicating the types of the observatons from one person;
%              1 if this person has only SP observations;
%              2 if this person has only RP observations;
%              3 if this person has both RP and SP observations.     
             
% The data should be sorted based on person's id, and for each person the RP observation is
% always first, then followed by the SP observations. The best way to get the organized data is to
% clean and sort it in a statistics package first (like STATA). Considering the example: the first 
% person has 1 RP observation and 3 SP observations; the second person has only 1 RP observation;
% the RP model is multinomial choice with 3 alternatives, and the SP model is binary;
% the first person is from RPDATA1 and the second person is from RPDATA2.  
% then the first data should look like:

% PID   CHOICES   YVEC   XFIXED   XRANDOM   RPDATA1  RPDATA2 	RP	SP
%  1	  1        0		      		1      0         1	0	
%  1      2        1			        1      0         1      0 
%  1      3        0                            1      0         1      0
%  1      0        0                            0      0         0      1
%  1      0        1                            0      0         0      1
%  1      0        1                            0      0         0      1 
%  2      1        0                            0      1         1      0  
%  2      2        0                            0      1         1      0
%  2      3        1                            0      1         1      0

% So in our setting, the RP observations are always in long format, whereas the SP observations
% are always in utility difference. There is only one row for each SP observation since SP model
% is binary.  

% Here is the example to load the data and allocate corresponding columns to above macros 
load mydata1.txt;
PID     = mydata1(:,1);
CHOICE  = mydata1(:,2);
YVEC    = mydata1(:,3);
XFIXED  = mydata1(:,4:(NFC + 3) );
XRANDOM = mydata1(:,(NFC + 3 + 1):(NFC + 3 + NNC) );
RPDATA1 = mydata1(:,(NFC + 3 + NNC + 1) );
RPDATA2 = mydata1(:,(NFC + 3 + NNC + 2));
RP      = mydata1(:,(NFC + 3 + NNC + 3));
SP      = mydata1(:,(NFC + 3 + NNC + 4));

% The second data should look like (in the case of random sampling)

% TIMES		WEIGHTS		DATAKINDS
%  6	      	  1		    3	
%  3		  1                 2
	
load mydata2.txt;
TIMES     = mydata2(:,1);
WEIGHTS   = mydata2(:,2);
DATAKINDS = mydata2(:,3);

%%% For the example of creating such a data from SATA, please read the SATA do file:
%%% getdata.do, which shows how to get data for our paper.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%% You do not need change the following parts %%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%% This section implements the random draws %%%%%%%%%%%%%%%%%%%%%%%

% Initialize the prime numbers used in Halton draws 
PRIME = [2;3;5;7;11;13;17;19;23;29;31;37;41;43;47;53;59;61;67;71;73;79;83;89;97;101];

% Generate Normal Random Numbers Using Halton Sequences
if NNC>0 & HALTON == 1
	% NERROR stores the random draws. For each random coefficient, we need REPTITION draws; 
	NERROR = zeros(sum(TIMES), NNC*REPTITION); 
	temp = zeros(NUSERS, NNC*REPTITION);
	for i = 1:NNC;
		gstart = (i-1)*REPTITION + 1;
		gend = (i-1) * REPTITION + REPTITION;
      		seed = PRIME(i);
      		% halton.m is the subroutine for generating halton sequence 
       		hdraw = halton( NUSERS*REPTITION, 0, seed, 1);
       		hdraw = transpose(hdraw);
       		for j = 1:NUSERS;
       			temp(j,gstart:gend) =...
            		hdraw( 1, ((j-1)*REPTITION + 1) : ((j-1)*REPTITION+REPTITION) );
       		end;
		clear hdraw;
	end;
	clear gstart gend;
	
	ini=0;
      	for i = 1:NUSERS;
       		gstart = ini+1;
       		gend = ini+TIMES(i);
       		NERROR(gstart:gend,:) = kron( temp(i,:), ones(TIMES(i),1) );
       		ini=ini+TIMES(i);
       	end;
       	clear temp;
   
end;

% Generate Normal Random Draws Using Pseudo-Random Draws
if NNC > 0 & HALTON == 0
	%randn('state', 8157);
    	NERROR = zeros( sum(TIMES), NNC*REPTITION);
    	temp = randn(NUSERS, NNC*REPTITION);
    	ini=0;
      	for i = 1:NUSERS;
       		gstart = ini+1;
       		gend = ini+TIMES(i);
       		NERROR(gstart:gend,:) = kron( temp(i,:), ones(TIMES(i),1) );
       		ini=ini+TIMES(i);
       	end;
       	clear temp;
end;

%%%%%%%%%%%%%%%%%% Initialize the parameters %%%%%%%%%%%%%%%%%%
% You can specify the initial values in a text editor, and then load the file here, or you can 
% specify at here directly.  
load b0.txt;

%%%%%%%%%%%%%%%%%% Optimize %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set options
options=optimset('TolFun',1*10^-3,'Display','iter','MaxFunEvals',300000000); 

% Find maximum of simulated log-likelihood function 
[beta,fval,exitflag,output,grad,hessian]=fminunc('likelihood',b0, options);

